// COPYRIGHT (c) 2002 PLOTNIX, INC. All rights reserved

/*
 * $Header$
 * $Revision$
 * $Date$
 *
 * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
 * reserved.
 */
package com.plotnix.jex;

/**
 * Variables provide access to a global set of values accessible via 
 * expressions. To use a custom implementation of this interface, 
 * pass it to the {@link Context#setVariables Context.setVariables()}
 * method
 * <p>
 * Note that a variable may be defined with a null value
 * 
 * @author Dmitri Plotnikov
 * @version $Revision$
 */
public interface Variables {

    /**
     * Returns true if the specified variable is declared.
     * 
     * @param varName a variable name
     * @return true if the variable exists
     */
    boolean isDeclaredVariable(String varName);

    /**
     * Returns the value of the specified variable.
     * Throws IllegalArgumentException if there is no such variable.
     * 
     * @param varName a variable name
     * @return the value of the variable
     */
    Object getVariable(String varName);

    /**
     * Defines a new variable with the specified value or modifies
     * the value of an existing variable. May throw 
     * UnsupportedOperationException.
     * 
     * @param varName a variable name
     * @param value the new value of the variable, which can be null
     */
    void declareVariable(String varName, Object value);
}