// COPYRIGHT (c) 2002 PLOTNIX, INC. All rights reserved

package com.plotnix.jex.adapters;

import java.util.Iterator;

import com.plotnix.jex.Compiler;
import com.plotnix.jex.Context;
import com.plotnix.jex.Expression;
import com.plotnix.jex.Interpreter;
import com.plotnix.jex.IterationExpression;
import com.plotnix.jex.VariableExpression;

/**
 * A wrapper for a compiler that makes it behave like an interpreter.
 * Each expression evaluation method first compiles the expression
 * and then invokes the resulting compiled expression.
 * <p>
 * The syntax checking methods invoke the compiler and throw away
 * the resulting compiled expression.
 *
 * @author Dmitri Plotnikov
 * @version $Revision$
 */
public class CompilingInterpreter extends Interpreter {

    private Compiler compiler;

    /**
     * @param compiler is the compiler that needs to be wrapped
     */    
    public CompilingInterpreter(Compiler compiler){
        this.compiler = compiler;
    }
    
    /**
     * @see Interpreter#evaluate(Context, String, Object[])
     */
    public Object evaluate(Context context, String expression, 
            Object[] arguments) {
        Expression expr = compiler.compileExpression(expression);
        return expr.evaluate(context, arguments);
    }

    /**
     * @see Interpreter#evaluate(Context, String, Object[], Class)
     */
    public Object evaluate(Context context, String expression, 
            Object[] arguments, Class valueType) {
        Expression expr = compiler.compileExpression(expression);
        return expr.evaluate(context, arguments, valueType);
    }

    /**
     * @see Interpreter#iterate(Context, String, Object[])
     */
    public Iterator iterate(Context context, String expression, 
            Object[] arguments) {
        IterationExpression expr = compiler.
            compileIterationExpression(expression);
        return expr.iterate(context, arguments);
    }

    /**
     * @see Interpreter#assign(Context, String, Object, Object[])
     */
    public void assign(Context context, String expression, 
            Object value, Object[] arguments) {
        VariableExpression expr = compiler.
                compileVariableExpression(expression);
        expr.assign(context, value, arguments);
    }
    
    /**
     * @see Interpreter#validateExpression(String)
     */
    public void validateExpression(String expression){
        compiler.compileExpression(expression);
    }

    /**
     * @see Interpreter#validateIterationExpression(String)
     */
    public void validateIterationExpression(String expression){
        compiler.compileIterationExpression(expression);
    }
    
    /**
     * @see Interpreter#validateVariableExpression(String)
     */
    public void validateVariableExpression(String expression){
        compiler.compileVariableExpression(expression);
    }

}
