// COPYRIGHT (c) 2002 PLOTNIX, INC. All rights reserved

package com.plotnix.jex.adapters;

import java.util.Iterator;

import com.plotnix.jex.Interpreter;
import com.plotnix.jex.Context;
import com.plotnix.jex.Expression;
import com.plotnix.jex.IterationExpression;
import com.plotnix.jex.VariableExpression;

/**
 * A wrapper for an expression string associated with an interpreter.
 * When asked to evaluate itself, an InterpreterExpression
 * passes the expression string to the interpreter for evaluation.
 *
 * @author Dmitri Plotnikov
 * @version $Revision$
 */
public class InterpretedExpression
    implements Expression, IterationExpression, VariableExpression {
        
    private Interpreter interpreter;
    private String expression;

    /**
     * @param interpreter is the interpreter for the expression language
     * @param expression is the expression to be interpreted
     */    
    public InterpretedExpression(Interpreter interpreter, String expression){
        this.interpreter = interpreter;
        this.expression = expression;
    }
    
    /**
     * @see Expression#evaluate(Context, Object[])
     */
    public Object evaluate(Context context, Object[] arguments) {
        return interpreter.evaluate(context, expression, arguments);
    }

    /**
     * @see Expression#evaluate(Context, Object[], Class)
     */
    public Object evaluate(Context context, Object[] arguments, 
            Class valueType) {
        return interpreter.
            evaluate(context, expression, arguments, valueType);
    }

    /**
     * @see IterationExpression#iterate(Context, Object[])
     */
    public Iterator iterate(Context context, Object[] arguments) {
        return interpreter.iterate(context, expression, arguments);
    }

    /**
     * @see VariableExpression#assign(Context, Object, Object[])
     */
    public void assign(Context context, Object value, Object[] arguments) {
        interpreter.assign(context, expression, value, arguments);
    }

}
