// COPYRIGHT (c) 2002 PLOTNIX, INC. All rights reserved

package com.plotnix.jex.bexl;

import java.util.Iterator;

import org.apache.commons.beanutils.PropertyUtils;

import com.plotnix.jex.Context;
import com.plotnix.jex.util.TypeUtils;
import com.plotnix.jex.util.ValueUtils;

/**
 * An Interpreter for the expression language introduced by
 * Jakarta Commons BeanUtils.  It understands properties,
 * indexing, dot-separated paths, mapped properties, DynaBeans.
 * See <a href="http://jakarta.apache.org/commons/beanutils/api/org/apache/commons/beanutils/PropertyUtils.html"
 * target="ref">
 * PropertyUtils</a> for the complete description of the language.
 *
 * @author Dmitri Plotnikov
 * @version $Revision$
 */
public class Interpreter extends com.plotnix.jex.Interpreter {
    
    /**
     * Invokes PropertyUtils.getProperty(target, expression), where
     * target is passed as the first and only argument.
     * 
     * @see com.plotnix.jex.Interpreter#evaluate(Context, String, Object[])
     */
    public Object evaluate(Context context, String expression, 
            Object[] arguments){
        if (arguments == null || arguments.length != 1){
            throw new RuntimeException("Incorrect number of arguments; " +
               "should be 1.");
        }
        Object target = arguments[0];
        try {
            return PropertyUtils.getProperty(target, expression);
        }
        catch (Exception ex){
           // TBD: wrap
           throw new RuntimeException("Cannot get property " + expression + 
                ex.getMessage());
        }
    }

    /**
     * Invokes PropertyUtils.getProperty(target, expression), where
     * target is passed as the first and only argument.
     * 
     * @see com.plotnix.jex.Interpreter#evaluate(Context, String, Object[], Class)
     */
    public Object evaluate(Context context, String expression, 
            Object[] arguments, Class valueType){
        Object result = evaluate(context, expression, arguments);
        if (!TypeUtils.canConvert(result, valueType)){
            throw new RuntimeException("Cannot convert " + result + " to " +
                valueType);
        }        
        return TypeUtils.convert(result, valueType);
    }

    /**
     * Evaluates the expression, treats the result as a collection and
     * returns an iterator of that collection.
     * 
     * @see com.plotnix.jex.Interpreter#iterate(Context, String, Object[])
     */
    public Iterator iterate(Context context, String expression, 
            Object[] arguments){
        Object collection = evaluate(context, expression, arguments);
        return ValueUtils.iterate(collection);
    }
    
    /**
     * Invokes PropertyUtils.setProperty(target, expression, value), where
     * target is passed as the first and only argument.
     * 
     * @see com.plotnix.jex.Interpreter#assign(Context, String, Object, Object[])
     */
    public void assign(Context context, String expression, 
            Object value, Object[] arguments){
        if (arguments == null || arguments.length != 1){
            throw new RuntimeException("Incorrect number of arguments; " +
               "should be 1.");
        }
        Object target = arguments[0];
        try {
            PropertyUtils.setProperty(target, expression, value);
        }
        catch (Exception ex){
           // TBD: wrap
           throw new RuntimeException("Cannot get property " + expression + 
                ex.getMessage());
        }
    }
}
