// COPYRIGHT (c) 2002 PLOTNIX, INC. All rights reserved

package com.plotnix.jex.javascript;

import java.io.IOException;
import java.io.StringReader;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.Script;

import com.plotnix.jex.Expression;
import com.plotnix.jex.IterationExpression;
import com.plotnix.jex.VariableExpression;



/**
 * Integrates ECMA-Script (aka JavaScript) with Jex. 
 * See <a href="http://www.mozilla.org/rhino/">Mozilla Rhino</a>
 * 
 * @author Dmitri Plotnikov
 * @version $Revision$ $Date$
 */
public class Compiler extends com.plotnix.jex.Compiler {
    
    /**
     * @see com.plotnix.jex.Compiler#compileExpression(String)
     */
    public Expression compileExpression(String expression) {
        Expression expr = null;
        Context cx = Context.enter();
        StringReader sr = new StringReader(expression);
        try {
            Script script = cx.compileReader(null, sr, "source", 1, null);
            expr = new JavaScriptValueExpression(script);
        }
        catch (IOException ex){
            ex.printStackTrace();   // This won't happen
        }
        Context.exit();

        return expr;
    }

    /**
     * @see com.plotnix.jex.Compiler#compileIterationExpression(String)
     */
    public IterationExpression compileIterationExpression(String expression) {
        IterationExpression expr = null;
        Context cx = Context.enter();
        StringReader sr = new StringReader(expression);
        try {
            Script script = cx.compileReader(null, sr, "source", 1, null);
            expr = new JavaScriptIterationExpression(script);
        }
        catch (IOException ex){
            ex.printStackTrace();   // This won't happen
        }
        Context.exit();

        return expr;
    }

    /**
     * @see com.plotnix.jex.Compiler#compileVariableExpression(String)
     */
    public VariableExpression compileVariableExpression(String expression) {
        VariableExpression expr = null;
        Context cx = Context.enter();
        StringReader sr = new StringReader(expression + 
                " = " + JavaScriptVariableExpression.JS_PSEUDO_VARIABLE);
        try {
            Script script = cx.compileReader(null, sr, "source", 1, null);
            expr = new JavaScriptVariableExpression(script);
        }
        catch (IOException ex){
            ex.printStackTrace();   // This won't happen
        }
        Context.exit();

        return expr;
    }
}
